%%%%%%%%%%%%%%%%%%%%%% cp7_3.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/9/99 HS
%----------------------------------------------------
%   .M file for analyzing the hydro-turbine system 
%      example for Comprehensive Problem CP7.3
%----------------------------------------------------
clear,clf
disp('Comprehensive Problem CP7.3')
disp('Find the step response performance of a proportional controller')

hydro  % read data
disp('*****>'), pause

 disp('Closed-loop system transfer function for ')
 disp('...proportional control Kp = 0.5')
 G = tf(numG,denG)
 T = feedback(0.5*G,1)
 disp('*****>'), pause 

 w = logspace(-2,2,100)';
 bode(T,w)
 title('CP7.3: Frequency response of closed-loop transfer function')
 keyboard
 [magT,phT] = bode(T,w);        			 % frequency response
 magTdB = 20*log10(magT);               % gain in dB
 lfgT = dcgain(T)          			 % dc gain
 lfgT = 20*log10(lfgT);
 BW = bwcalc(magTdB,w,lfgT)             % bandwidth
 [Mp,i] = max(magTdB)                   % maximum freq. resp.
 wr = w(i)                                
 disp('*****>');pause
 
 disp('Closed-loop system sensitivity function for ')
 disp('...proportional control Kp = 0.5')
 S = feedback(1,0.5*G) 
 bode(S)
 title('CP7.3: Frequency response of closed-loop sensitivity function')
 keyboard
 [magS,phS] = bode(S,w);                          % sensitivity
 magS_dB = 20*log10(magS);
 [maxS,imaxS] = max(magS_dB)
 w_maxS = w(imaxS)
 clf
 semilogx(w,magTdB(:),'-',w,magS_dB(:),'--'), grid
 title('CP7.3: Sensitivity and transfer function response')
 xlabel('Frequency (rad/sec)')
 ylabel('Gain dB')
 text(0.12,-12,'|T(jw)|')
 text(0.12,-2,'|S(jw)|') 
 keyboard

 disp('Step response performance measures')  
 t = [0:0.05:20]';
 y = step(T,t);
 clf, plot(t,y); grid
 ylabel('Amplitude');xlabel('Time (sec)')
 title('CP7.3: Step response of hydro-turbine system')

 disp('compute time-domain measures - use 1 as steady-value for the 3rd') 
 disp('... argument for tstats.') 
 lfgT = dcgain(T)               
 [Mo,tp,tr,ts,ess] = tstats(t,y,1);
 text(2,-0.60,['overshoot = ' num2str(Mo) ' %'])
 text(2,-0.45,['time to overshoot = ' num2str(tp) ' s'])
 text(2,-0.30,['rise time = ' num2str(tr) ' s'])
 text(2,-0.15,['2pct settling time = ' num2str(ts) ' s'])
 text(2,0.0,['steady-state error = ' num2str(ess) ' %'])

disp('end of Comprehensive Example 7.3')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

