%%%%%%%%%%%%%%%%%%%%%% ex8_2.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
%	             PI control
%----------------------------------------------------
clear,clf
disp('Example 8.2: PI control')
yes = 'y'; no = 'n'; 
repeat = yes;
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
denGp = conv([tauP1 1],[tauP2 1]);
Gp    = tf(Kproc,denGp)
H     = tf(1,[tau_sen 1])
%---------- open-loop transfer function
GpH   = Gp*H;
%============== Part (a): sweep KP for various KI values
disp('sweep KP for various KI values')
%----- CL step response to reference input
t = [0:0.02:10];
while(repeat == yes),	% allow multiple runs
  disp(' ')
  disp('Book uses KI = 0, 0.1, 0.25, 0.5, 1.0, & 2.0')
  disp(' ')
  KI = input('enter integral gain KI ==> ');
  disp(' ')
  KI_str=num2str(KI);
  for KP = [0.3:0.2:1.7];
    Kc = tf(KP*[1 KI],[1 0]);
    T = feedback(Kc*Gp,H,-1);
    ys = step(T,t);
    plot(t,ys)
    if ishold ~=  1, hold on, end
  end	% of 'for'
  xlabel('Time (sec.)'),ylabel('Amplitude')
  grid;
  title('Ex 8.2: CL step resp, P + I control, Fig. 8.3')
  % plot(t,0.98*ones(size(t)),'--')
  % plot(t,1.02*ones(size(t)),'--')
  text(5.0,0.25,'KP = 0.3,0.5, ... ,1.7')
  text(5.0,0.15,['KI = ' KI_str])
  hold off
  text(1.40,1.26,'KP = 1.7')
  text(3.0,0.67,'KP = 0.3')
  keyboard
  reply = input('repeat for another value of KI(yes/no)? ==> ')
  if reply == yes, repeat = yes;
   else repeat = no;
  end	% of 'if'
end	% of 'while' 
%
%=============== Part (b): root locus =============
disp('Part (b): root locus')
KI = 0.5	% selected based on 2% settling time
KI_str=num2str(KI);
%--- do root locus for KP = 1
disp('first with default area')
Kc = tf(KP*[1 KI],[1 0]);
rlocus(Kc*Gp*H)
grid
axis equal
title('Ex 8.2: root locus for PI control')
text(-40,-25,['KI = ' KI_str])
keyboard
disp('use smaller area for details near origin')
axis([-8 2 -2 8])
text(-6,1,['KI = ' KI_str])
keyboard
[kk,polesCL] = rlocfind(Kc*Gp*H)
keyboard

%==== Part (c): reference & disturbance step responses ===
disp('Part (c): reference & disturbance step responses')
disp('setting KP = 0.6')
KP = 0.6;	% selected for 10% overshoot
KP_str=num2str(KP);
Kc = tf(KP*[1 KI],[1 0]);
%----- closed-loop step response to reference input
[T_ref] = feedback(Gp*Kc,H,-1);
t = [0:0.02:10];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
%-------- CL step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Kc*H,+1);
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
axis([0 10 0 1.2])
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 8.2: CL step responses, PI control, Fig. 8.5')
text(6.0,0.55,['KP = ' KP_str])
text(6.0,0.45,['KI = ' KI_str])
text(7.0,1.05,'reference')
text(6.0,0.15,'disturbance')
disp('end of Example Problem 8.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





