%%%%%%%%%%%%%%%%%%%%% wif8_3c.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/22/99 HS
%----------------------------------------------------
%  This MATLAB file runs the What-if problem after 
%  						Example 8.3
%----------------------------------------------------
clear,clf
disp('Wif 8.3c: PD control')
y = 'y'; n = 'n'; 
repeat = y;
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
numGp = Kproc;
denGp = conv([tauP1 1],[tauP2 1]);
H     =tf(1,[tau_sen 1]);
%
%============== Part (a): sweep KP for various KI and KD values
disp('sweep KP for various KD values')
disp('KI = 0 for PD control')
disp(' ')
KI = 0;
KI_str=num2str(KI);
denGc = [1 0];
denG = conv(denGc,denGp);
%----- CL step response to reference input
t = [0:0.01:2]';
while(repeat == y),	% allow multiple runs
   disp('A reasonable KD value is 0.5')
   KD = input('enter derivative gain KD ==> ');
   disp(' ')
   KD_str=num2str(KD);
   for KP = [1:1:10];
    numKc = KP*[KD 1 KI];   % numerator of PID controller
    numGK = conv(numKc,numGp);
    GK = tf(numGK,[denGp 0]);
    T = feedback(GK,H,-1);
    ys = step(T,t);
    plot(t,ys)
    if ishold ~=  1, hold on, end
  end	% of 'for'
  xlabel('Time (sec.)'),ylabel('Amplitude')
  grid;
  title('Wif 8.3c: CL Step Response, PD control')
  text(1.0,0.35,'KP = 1,2,3,...,10')
  text(1.0,0.15,['KD = ' KD_str])
  text(0.25,1.16,'KP = 10')
  text(0.7,0.65,'KP = 1')
  hold off
  keyboard
  reply = input('repeat for new values of KD (y/n)? ==> ');
  if reply == y, repeat = y;
   else repeat = n;
  end	% of 'if'
end	% of 'while' 
%
%=============== Part (b): root locus =============
disp('Part (b): root locus')
%-- Note: can not use series cmd because Gc(s) is not proper
%   Also, can not use: numRL = conv([KD 1 KI],numGpH);
%      to form numerator because  numGpH has been padded
%      with 3 leading zeros, causing a bogus error message
%      from the MATLAB routine tfchk
    numKc = KP*[KD 1 KI];   % numerator of PID controller
    numGK = conv(numKc,numGp);
    GK = tf(numGK,[denGp 0]);
    disp('draw root locus with default area')
    rlocus(GK*H); grid
axis equal
title('Wif 8.3c: open-loop, PD control')
text(20,-15,['KD = ' KD_str])
keyboard
disp('redraw locus with smaller area')
axis([-15 5 -10 10])
title('Wif 8.3c: open-loop, PD control')
text(0,-6,['KD = ' KD_str])
keyboard
disp('redraw locus with still smaller area')
axis([-2.5 0.5 -1.2 1.2])
title('Wif 8.3c: open-loop, PD control')
text(-0.2,-1.0,['KD = ' KD_str])
keyboard
[kk,polesCL] = rlocfind(GK*H)

%==== Part (c): reference & disturbance step responses ===
disp('Part (c): reference & disturbance step responses')
disp('A reasonable value of KP is 7.0')
disp(' ')
KP = input('enter proportional gain KP ==> ');
KP_str=num2str(KP);
numKc = KP*[KD 1 KI];   % numerator of PID controller
numGK = conv(numKc,numGp);
GK = tf(numGK,[denGp 0]);
%----- closed-loop step response to reference input
T_ref = feedback(GK,H,-1);
t = [0:0.01:3];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
Gp = tf(numGp,denGp);
HK = tf(-numKc,[tau_sen 1 0]);
%-------- CL step response to disturbance input
%--- use +1 for feedback gain because neg sign is in HK(s)
T_dist = feedback(Gp,HK,+1);
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Wif 8.3c: CL step responses, PD control')
text(2.0,0.50,['KP = ' KP_str])
text(2.0,0.30,['KD = ' KD_str])
text(1.0,1.05,'reference')
text(1.0,0.15,'disturbance')
disp('end of What If 8.3c')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%