%%%%%%%%%%%%%%%%%%%%%% ex9_5.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
clear,clf
disp('Lead-Lag control based on PID design')
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]))
H  = tf(1,[tau_sen 1])
GpH = Gp*H
%---------- open-loop transfer function
GpH = Gp*H
disp('Part (a): use PID gains to calc zeros')
KI = 0.5
KD = 0.4
KI_str = num2str(KI);
KD_str = num2str(KD);
disp('Lead-lag zeros match PID zeros')
%------- calculate lead-lag parameters
Alpha = 20; % ratio of lead-lag zero to pole
Alpha_str = num2str(Alpha);
Zpid = roots([KD  1  KI])   % both should real & negative
Zlead = min(Zpid)	% furthest to left
Zlag = max(Zpid)	% furthest to right
Zlead_str=num2str(Zlead);
Zlag_str=num2str(Zlag);
%----- build lead-lag without any gain to do root locus
numldlg = conv([1 -Zlag],[1 -Zlead]); % lead-lag without gain
denldlg = conv([1 -Zlag/Alpha],[1 -Alpha*Zlead]); 
G_ldlg  = tf(numldlg,denldlg)
disp('*******>');pause

%========= Part (b) root locus, find max gain for stability
rlocus(GpH*G_ldlg); grid
axis([-60 10 -35 35])	% 
title('Ex 9.5: open-loop, lead-lag controller')
text('Units','normalized',...
	'Position',[0.05,0.88],'String',['Zlag = ' Zlag_str])
text('Units','normalized',...
	'Position',[0.05,0.81],'String',['Zlead = ' Zlead_str])
text('Units','normalized',...
	'Position',[0.05,0.74],'String',['Alpha = ' Alpha_str])
text('Units','normalized',...
	'Position',[0.05,0.67],...
	'String',['based on KI = ' KI_str])
text('Units','normalized',...
	'Position',[0.10,0.60],'String',['&  KD = ' KD_str])
axis('equal')
keyboard
[kk,polesCL] = rlocfind(GpH*G_ldlg)
keyboard
disp('redraw with smaller area ')
axis([-3 1 -2 2])
keyboard
%----- OL frequency response + gain & phase margins
margin(GpH*G_ldlg)
text('Units','normalized',...
	'Position',[0.05,0.80],'String','Example 9.5')
text('Units','normalized',...
	'Position',[0.05,0.65],'String',['Zlag = ' Zlag_str])
text('Units','normalized',...
	'Position',[0.05,0.55],'String',['Zlead = ' Zlead_str])
text('Units','normalized',...
	'Position',[0.05,0.45],'String',['Alpha = ' Alpha_str])
text('Units','normalized',...
	'Position',[0.05,0.35],...
	'String',['based on KI = ' KI_str])
text('Units','normalized',...
	'Position',[0.10,0.25],...
	'String',['&  KD = ' KD_str])
subplot(111)

disp('Part (c) gain sweep')
%------------- closed-loop step response to reference input
t = [0:0.01:2];
for Kldlg = [40:10:120];
  T  = feedback(Gp*Kldlg*G_ldlg,H,-1);
  ys = step(T,t);
  plot(t,ys)
  if ishold ~=  1, hold on, end
end
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 9.5: CL responses,lead-lag control, Fig. 9.13')
text(1.0,0.65,'Kldlg = 40, 50,...,120')
text(1.0,0.55,['Lag zero = ' Zlag_str])
text(1.0,0.45,['Lead zero = ' Zlead_str])
text(1.0,0.35,['Alpha = ' Alpha_str])
text(1.0,0.25,['based on KI = ' KI_str])
text(1.0,0.15,['      &  KD = ' KD_str])
text(0.28,1.30,'Kldlg = 120')
text(0.28,0.70,'Kldlg = 40')
hold off
keyboard

disp('Part (d) Step responses')
Kldlg = 70
Kldlg_str = num2str(Kldlg);
%==========  CL step responses for Fig. 9.14 =====
%---- build CL system for ref input with sensor in fdbk path
T_ref = feedback(Gp*Kldlg*G_ldlg,H,-1);
t = [0:0.01:3];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
%--- dist input: fwd path = Gp; fdbk path = -H*Gc
%------ closed-loop step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Kldlg*G_ldlg*H,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 9.5: CL ref & dist step responses Fig. 9.14')
text(1.0,0.80,['Kldlg = ' Kldlg_str])
text(1.0,0.70,['Zlead = ' Zlead_str])
text(1.0,0.60,['Zlag = ' Zlag_str])
text(1.0,0.50,['Alpha = ' Alpha_str])
text(1.0,1.05,'reference')
text(1.0,0.20,'disturbance')
axis([0 3 0 1.2])
disp('end of Example Problem 9.5')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%