%%%%%%%%%%%%%%%% fig9_14.m %%%%%%%%%%%%%%%
%
%      2nd order plant with 2 real poles with lead-lag control
% 	runs reference & disturbance step response for
% 	Jan. 99
%  last revision:  1/8/99  JHC
%----------------------------------------
clear;clf
set(gcf,'DefaultAxesFontSize',16)
set(gcf,'DefaultTextFontSize',16)
disp('fig 9.14: Lead-Lag control, based on PID ')
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]))
H  = tf(1,[tau_sen 1])
%---- specify controller
Kldlg = 70;
Alpha = 20; Zlag = -0.691; Zlead = -1.809;
Zlag_str=num2str(Zlag);
Zlead_str=num2str(Zlead);
Kldlg_str=num2str(Kldlg);
Alpha_str = num2str(Alpha);
%----------
numldlg = conv([1 -Zlag],[1 -Zlead]); % lead-lag without gain
denldlg = conv([1 -Zlag/Alpha],[1 -Alpha*Zlead]); 
G_ldlg  = tf(numldlg,denldlg);
%==========  CL step responses for Fig. 9.14 =====
%---- build CL system for ref input with sensor in fdbk path
T_ref = feedback(Gp*Kldlg*G_ldlg,H,-1);
t = [0:0.01:3];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
%--- dist input: fwd path = Gp; fdbk path = -H*Gc
%------ closed-loop step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Kldlg*G_ldlg*H,+1)
y_dist = step(T_dist,t);

%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
% title('fig 9.14: CL step resp,lead-lag control based on PID')
text(2.0,0.75,['Kldlg = ' Kldlg_str])
text(2.0,0.68,['Zlag = ' Zlag_str])
text(2.0,0.61,['Zlead = ' Zlead_str])
text(2.0,0.54,['Alpha = ' Alpha_str])
text(1.0,1.05,'reference')
text(1.0,0.20,'disturbance')
axis([0 3 0 1.2])
print -deps fig9_14.eps
%-- reset fonts to defaults
set(gcf,'DefaultAxesFontSize',get(0,'DefaultAxesFontSize'));
set(gcf,'DefaultTextFontSize',get(0,'DefaultTextFontSize'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
