%%%%%%%%%%%%%%%% fig9_6.m %%%%%%%%%%%%%%%
% 	Jan. 99
%  last revision:  1/8/99  JHC
%----------------------------------------
clear;clf
set(gcf,'DefaultAxesFontSize',16)
set(gcf,'DefaultTextFontSize',16)
disp('fig 9.6: Lag control via Kuo''s algorithm')
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4; 
%---- final design
Klag = 12.25;
Zlag = -0.1572;
Alag = 11.64;
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]));
H  = tf(1,[tau_sen 1]);
GpH = Gp*H;
%------- OL frequency response, gain only
w = logspace(-3,2,100);
zdb = 0*w;
ph180 = -180*ones(length(w));
%------ gain-only
[mag_K,ph_K] = bode(Klag*GpH,w);
mag_K    = reshape(mag_K,100,1);
ph_K     = reshape(ph_K, 100,1);
mag_db_K = 20*log10(mag_K);
%
%---- Kuo's form for Gc(s)
Gc = tf(Klag*[-1/Zlag  1],[-Alag/Zlag  1]);	 
%---- open-loop transfer function of compensated system
GcGpH = Gc*GpH;
[mag_lag,ph_lag] = bode(GcGpH,w);
mag_lag = reshape(mag_lag,100,1);
ph_lag  = reshape(ph_lag ,100,1);
mag_db_lag = 20*log10(mag_lag);
%
%--------- now plot both sets
subplot(211)
semilogx(w,mag_db_lag,'-',w,mag_db_K,'--',w,zdb,'-');
grid;
axis([0.001 100  -100 50])
xlabel('Frequency (rad/sec)')
ylabel('Gain dB')
%title('fig 9.6: lag & P-only, Kuo''s method, Ex 9.2')
text(2.0,-25,'Lag')
text(2.0,25,'Proportional')
subplot(212)
semilogx(w,ph_lag,'-',w,ph_K,'--',w,ph180,'-');
grid
axis([0.001 100  -300 0])
xlabel('Frequency (rad/sec)')
ylabel('Phase deg')
text(0.02,-85,'Lag')
text(0.30,-25,'Proportional')
print -deps fig9_6.eps
subplot(111)	% reset plot window
%-- reset fonts to defaults
set(gcf,'DefaultAxesFontSize',get(0,'DefaultAxesFontSize'));
set(gcf,'DefaultTextFontSize',get(0,'DefaultTextFontSize'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%