%%%%%%%%%%%%%%%%%%%%%% one_blk.m %%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
clear,clf
activity = -1;    % force activity menu to appear
while (activity < 1) | (activity > 3),
  disp(' 1 <= Enter or modify a 1-block model')
  disp(' 2 <= Calculations')
  disp(' 3 <= Plots')
  activity = input('Select action, or enter 0 to quit....');

if activity == 0, disp('Completed single-block analysis'), return, end

if activity == 1,	% enter block model in either tf or zp form
model_form = -1;  % force model entry menu to appear
while (model_form < 1) | (model_form > 6),
  disp(' 1 <= Build LTI object from TF data')
  disp(' 2 <= Build LTI object from ZPK data')
  disp(' 3 <= Display numerator and denominator values')
  disp(' 4 <= Display ZPK values')
  disp(' 5 <= Create unity-feedback closed-loop model')
  disp(' 6 <= Activate keyboard')
  model_form = input...
    ('Select model form or enter 0 for top-level menu.....'); 

if (model_form == 0 | isempty(model_form)),
  activity = -1;   % return to activity menu
  break   % terminate "while model_form..." loop
end    % of " if model_form == 0"

%+++++++++++++++++++ enter model in TF form ++++++++++++++++++++
  if model_form == 1,
    disp(' ')
  disp('   **** Interactive entry of transfer function using TF data ****')
  disp(' ')
% remove any stale zeros, poles, & gain from workspace
clear zer zer_str zer_last pol pol_str pol_last gain gain_str gain_last
  disp('Enter coefficients of numerator and denominator polynomials.')
  disp('Have initialized with values from Example 2.3')
  %-----------------------------------------------------
  num_last = [3 2];	% numerator specified in Example 2.3
  den_last = [2 4 5 1];	% denominator specified in Example 2.3
  num_str = vec2str(num_last);	% string representation of numerator
  den_str = vec2str(den_last);	% string representation of denominator
  cmd = 'n';	% 'n' ==> model is not OK
  while cmd == 'n',
    disp(' ')
    % allow user to enter new values for H(s)
    num = input(['Enter numerator  "' num_str '"....']);
    if isempty(num), num = num_last; end   %  CR ==> use old value
    den = input(['Enter denominator  "' den_str '"....']);
    if isempty(den), den = den_last; end   %  CR ==> use old value
    num_str = vec2str(num);  % 
    den_str = vec2str(den);
    disp('Numerator and denominator polynomials of tf form are:')
    eval(['num = ' num_str]);
    eval(['den = ' den_str]);
    cmd = input('Model OK? (y|n)','s');
    num_last = num;    % save previous values in case want to reuse
    den_last = den;
    H = tf(num,den);
  end   % of "while" for "run another case"
  disp('Model has been entered as TF object')
  model_form = -1;   % force model_form menu to reappear
end  % of "if model_form ==1"

%++++++++++++++++ enter model in ZP form +++++++++++++++++++++
if model_form ==2,
disp(' ')
disp('   **** Interactive entry of transfer function using ZPK data ****')
disp(' ')
disp('Enter H(s) in terms of zeros, poles, and gain ')
disp('Have initialized with values from Example 3.1')
% remove any stale numerator & denominator from workspace
clear num num_str num_last pol pol_str pol_last
zer_last = [-2; -1.5];	% 2 real zeros 
pol_last = [-0.5; -0.2+0.6*i; -0.2-0.6*i; -8];	% 4 poles, 2 complex
gain_last = 2;	% gain 
zer_str = vec2str(zer_last)	% string repr of numerator with ';'
pol_str = vec2str(pol_last)	% string repr of denominator with ';'
gain_str = num2str(gain_last)	% string repr of gain
cmd = 'n';	% 'n' ==> model not OK
while cmd == 'n',
  disp(' ')
  % allow user to enter new values for H(s)
  zer = input(['Enter zeros as column vector "' zer_str '"....']);
  if isempty(zer), zer = zer_last; end   %  CR ==> use old value
  pol = input(['Enter poles as column vector "' pol_str '"....']);
  if isempty(pol), pol = pol_last; end   %  CR ==> use old value
  gain = input(['Enter gain "' gain_str '"....']);
  if isempty(gain), gain = gain_last; end   %  CR ==> use old value
  zer_str = vec2str(zer)  % 
  pol_str = vec2str(pol)
  gain_str = num2str(gain)
  disp('Zeros, poles, & gain of zp form are:')
  eval(['zer = ' zer_str]);
  eval(['pol = ' pol_str]);
  eval(['gain = ' gain_str]);
  cmd = input('Model OK? (y|n)','s');
  zer_last = zer;    % save previous values in case want to reuse
  pol_last = pol;
  gain_last = gain;
  H = zpk(zer,pol,gain);
end   % of "while cmd = n"
disp('Model has been entered in zero-pole-gain form')
  model_form = -1;   % force model_form menu to reappear
end   % of "if model_form == 2"

%++++++++++ Display ZPK values ++++++++++++++
  if model_form == 4,
     if (exist('H')),
       disp('Zeros, poles, and gain of H(s) are:')
       [zer,pol,gain] = zpkdata(H,'v')
       disp('********>');pause
     else
       disp...
         ('Need model defined as LTI object')
       disp('*******>');pause
     end    % of "if exist..."
     model_form = -1;   % force model_form menu to reappear
  end    % of "model_form == 4"

%+++++++++++ Display numerator and denominator values ++++++++++++++
  if model_form == 3,
    if (exist('H')),
       disp('Numerator and denominator of H(s) are:')
       [num,den] = tfdata(H,'v')
       disp('********>');pause
     else
       disp('Need model defined as LTI object')
       disp('*******>');pause
    end
    model_form = -1;   % force model_form menu to reappear
  end    % of "model_form == 3"

%+++++++++++++++ unity feedback closed-loop system +++++++
 if model_form == 5,
   if (exist('H')),
       disp('saving open-loop model as "HOL" ')
       disp('....because new closed-loop system will be "H"')
       disp('Numerator and denominator of closed-loop system are:')
       HOL = H;
       H = feedback(HOL,1)
       disp('*******>');pause
     else
       disp(' ')
       disp('require open-loop model')
       disp('******>');pause
    end
    model_form = -1;   % force model_form menu to reappear
  end    % of "model_form == 5"
%+++++++++++++++++ activate keyboard ++++++++++++++++++++
if model_form == 6 ,
  disp('keyboard is active to allow:')
  disp('  * examining workspace')
  disp('  * entry of externally-defined model')
  disp('  * use of any Matlab commands')
  disp('To resume, enter "return"')
  keyboard
  model_form = -1;   % return to model_form menu
end    % of "model_form == 6"
%-------
end   % of " while model_form....."
end   % of " if activity == 1"
%========== end of model creation ===========

%=========== calculations ===============
if activity == 2,
  disp('Select calculations to be done for model')
  calc_type = -1;  % force calculation type menu to appear
  while (calc_type < 1) | (calc_type > 6),
    disp(' 1 <= Numerator & denominator of H(s)')
    disp(' 2 <= Zeros, poles, and gain of H(s)')
    disp(' 3 <= Residues and poles of H(s)')
    disp(' 4 <= Display state-space matrix')
    disp(' 5 <= Damping ratio & natural freq of poles of H(s)')
    disp(' 6 <= examine work space')
    calc_type = input...
    ('Select type of calculation or enter 0 for top-level menu.....');

%+++++++++++++++++ no more calculations ++++++++++++++++++++
if (calc_type == 0 | isempty(calc_type)),
  disp('Completed calculations')
  activity = -1;   % return to activity menu
  break     % terminate "while calc_type...." loop
end    % of "calc_type == 0"

%++++++++++++++++++++ calculate num & den ++++++++++++++
  if calc_type == 1,
    if (exist('H')),
       disp('Numerator and denominator of H(s) are:')
       [num_calc,den_calc] = tfdata(H,'v')
       disp('********>');pause
     else
       disp('Need model as LTI object for this calculation')
       disp('*******>');pause
    end
    calc_type = -1;   % force calc_type menu to reappear
  end    % of "calc_type == 1"

%++++++++++++++++++++ calculate zeros, poles, and gain ++++++++
  if calc_type == 2,
     if (exist('H')),
       disp('Zeros, poles, and gain of H(s) are:')
       [zer_calc,pol_calc,gain_calc] = zpkdata(H,'v')
       disp('********>');pause
     else
       disp...
         ('Need model as LTI object for this calculation')
       disp('*******>');pause
     end    % of "if exist..."
     calc_type = -1;   % force calc_type menu to reappear
  end    % of "calc_type == 2"

%+++++++ calculate  Residues, poles, and "other part"+++++++++++
  if calc_type == 3,
     if (exist('H')),
        disp('Residues, poles, and "other part" of H(s) are:')
        [num,den] = tfdata(H,'v');
       [residues,poles,other] = residue(num,den)
     else
       disp('Need model as LTI object for this calculation')
     end   % of "if exist ..."
       disp('********>');pause
     calc_type = -1;   % force calc_type menu to reappear
  end    % of "calc_type == 3"

%+++++++++++++ calculate state-space model +++++++++++++
  if calc_type == 4,
     if (exist('H')),
        disp('ABCD matrices of state-space form of H(s) are:')
        [A,B,C,D] = ssdata(H)
      else
       disp('Need model as LTI object for this calculation')
     end   % of "if exist ..."
     disp('********>');pause
     calc_type = -1;   % force calc_type menu to reappear
  end    % of "calc_type == 4"

%++++++++++ calculate damping ratio & natural freq ++++++++
  if calc_type == 5,
     if (exist('H')),
        [wn,zeta] = damp(H);
       elseif (exist('pol'))  
        [wn,zeta] = damp(pol);    
       else
         disp('Require model as LTI object for this selection')
      end     % of "if exist.....
    disp('Undamped natural frequency & damping ratio are:')
    disp([wn zeta])
    disp('********>');pause
    calc_type = -1;   % force calc_type menu to reappear
  end    % of "calc_type == 5"

%+++++++++++++++++ activate keyboard ++++++++++++++++++++
if calc_type == 6 ,
  disp('keyboard is active to allow:')
  disp('  * examining workspace')
  disp('  * entry of externally-defined model')
  disp('  * use of any Matlab commands')
  disp('To resume, enter "return"')
  keyboard
  calc_type = -1;   % return to calc_type menu
end    % of "calc_type == 6"
%----------
end    % of "while calc_type ...."
end    % of "if activity == 2"

%============= end of calculations ===================

%=========== plots ===============
if activity == 3,
   if (exist('H'))
   else
       disp('Need model as LTI object for this calculation')
       return
   end   % of "if exist ..."
  disp('  ')
  disp('Select plots to be done for model')
  disp('When the plot appears the keyboard will be active')
  disp('You may enter a title, add grid lines, and make')
  disp('a hard copy of the plot')
  disp('To resume, return from "keyboard" command')
  disp('Enter "return" for Unix, use CTRL-Z for DOS')
  disp('  ')
  plot_type = -1;  % force plot menu to appear
  while (plot_type < 1) | (plot_type > 7),
    disp(' 1 <= Impulse response')
    disp(' 2 <= Step response')
    disp(' 3 <= Bode plot')
    disp(' 4 <= Nyquist plot')
    disp(' 5 <= Nichols plot')
    disp(' 6 <= Root locus')
    disp(' 7 <= Pole-zero map')
  plot_type = input...
    ('Select type of calculation or enter 0 for top-level menu.....');

%+++++++++++++++++ no more plots ++++++++++++++++++++
if (plot_type == 0 | isempty(plot_type)),
  disp('Completed plots')
  activity = -1;   % return to activity menu
  break    % terminate "while plot_type ..." loop
end    % of "plot_type == 0"

%++++++++++++++++++++ impulse response ++++++++++++++
  if plot_type == 1,
    impulse(H)
    grid; title('Impulse response')
     %-- add H(s) values to plot
     [num,den] = tfdata(H,'v');
     num_str = vec2str(num);
     den_str = vec2str(den);
     text('Units','Norm','Pos',[0.6 0.80],'String',...
                     		   ['num = ' num_str])
	  
	  text('Units','Norm','Pos',[0.6 0.75],'String',...
                     		   ['den = ' den_str])
    plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 1"
%++++++++++++++++++++ step response ++++++++
  if plot_type == 2,
      step(H)
      grid; title('Step response')
     %-- add H(s) values to plot
     text('Units','Norm','Pos',[0.6 0.80],'String',...
                     		   ['num = ' num_str])
     
	  text('Units','Norm','Pos',[0.6 0.75],'String',...
                     		   ['den = ' den_str])
     plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 2"


%+++++++ Bode plot +++++++++++
  if plot_type == 3,
     bode(H)
     title('Bode plot')
     %-- add H(s) values to plot
     text('Units','Norm','Pos',[0.6 0.80],'String',...
                     		   ['num = ' num_str])
     text('Units','Norm','Pos',[0.6 0.70],'String',...
                     		   ['den = ' den_str])
     %subplot(1,1,1)
     else
       disp...
     plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 3"


%+++++++++++++ Nyquist plot +++++++++++++
  if plot_type == 4,
     nyquist(H)
     grid; title('Nyquist plot')
     plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 4"

%++++++++++ Nichols plot ++++++++
  if plot_type == 5,
     nichols(H)
     grid; title('Nichols plot')
     %-- add H(s) values to plot
     text('Units','Norm','Pos',[0.6 0.80],'String',...
                     		   ['num = ' num_str])
     text('Units','Norm','Pos',[0.6 0.75],'String',...
                     		   ['den = ' den_str])
    plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 5"

%++++++++++ root locus plot ++++++++
  if plot_type == 6,
     disp('Use "rlocfind" command after plot has been drawn and')
     disp('keyboard is active to find gains at various points')
     disp('Syntax is "[k,p]=rlocfind(num,den)" where ')
     disp('    k = gain at selected point')
     disp('    p = closed-loop poles for that value of gain')
     rlocus(H)
     grid; title('Root Locus')
     %-- add H(s) values to plot
     text('Units','Norm','Pos',[0.6 0.80],'String',...
                     		   ['num = ' num_str])
     text('Units','Norm','Pos',[0.6 0.75],'String',...
                     		   ['den = ' den_str])
    plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 6"

%++++++++++ pole-zero map ++++++++
if plot_type == 7,
       pzmap(H)
       grid; title('Pole-zero map from TF form')
       %-- add H(s) values to plot
       text('Units','Norm','Pos',[0.6 0.80],'String',...
                     		   ['num = ' num_str])
       text('Units','Norm','Pos',[0.6 0.75],'String',...
                     		   ['den = ' den_str])
    plot_type = -1;   % force plot_type menu to reappear
  end    % of "plot_type == 7"
%-----
end    % of "while plot_type ...."
end    % of "if activity == 3"
%================================
end  % of "while" for activity menu
%%%%%%%%%%%%%%%%%% end of one_blk.m%%%%%%%%%%%%%%%%%
